package ru.skipy.logging.jmx;

import org.apache.log4j.Level;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.apache.log4j.xml.DOMConfigurator;

import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.lang.management.ManagementFactory;
import java.net.InetAddress;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * JMX Log4J management example
 *
 * @author Eugene Matyushkin aka Skipy (<a href="http://www.skipy.ru">www.skipy.ru</a>)
 */
public class Main extends JFrame {

  /**
   * Logger instance
   */
  private static final Logger logger = Logger.getLogger(Main.class);
  
  /**
   * MBean object name
   */
  public static final String OBJECT_NAME = "ru.skipy.logging.jmx:type=Log4J JMX Manager";

  /**
   * Creates test application frame
   */
  Main (){
    super("Log4J Management MBean example");
    JButton btn = new JButton("Press to test log...");
    btn.addActionListener(new ActionListener(){
      @Override
      public void actionPerformed(ActionEvent e) {
        test();
      }
    });
    JPanel cp = new JPanel(new BorderLayout(5,5));
    setContentPane(cp);
    cp.add(btn, BorderLayout.CENTER);
    JLabel lbl = new JLabel("<html>Use JConsole to connect MBean<br>Process ID: <font color=\"blue\">"+getPID()+
        "</font><br>MBean Name: <font color=\"blue\">"+OBJECT_NAME+"</font></html>");
    cp.add(lbl, BorderLayout.NORTH);
    cp.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
    setSize(400,150);
    setLocationRelativeTo(null);
    setDefaultCloseOperation(EXIT_ON_CLOSE);
  }

  /**
   * Starts application
   *
   * @param args application arguments
   */
  public static void main(String[] args) throws Exception{
    initMBean();
    new Main().setVisible(true);
  }

  /**
   * Initializes MBean with the object name {@link #OBJECT_NAME}
   *
   * @throws Exception if any error occures
   */
  private static void initMBean() throws Exception{
    MBeanServer mbs = ManagementFactory.getPlatformMBeanServer();
      ObjectName name = new ObjectName(OBJECT_NAME);
      Log4JManager mbean = new Log4JManager();
      mbs.registerMBean(mbean, name);
  }

  /**
   * Testing all log levels
   */
  private void test(){
    System.out.println("Testing all levels:\n====================");
    System.out.println("Trace: ");
    if (logger.isTraceEnabled()){
        logger.trace("TRACE message");
    }
    System.out.println("Debug: ");
    if (logger.isDebugEnabled()){
        logger.debug("DEBUG message");
    }
    System.out.println("Info: ");
    logger.info("INFO message");
    System.out.println("Warn: ");
    logger.warn("WARN message");
    System.out.println("Error: ");
    logger.error("ERROR message");
    System.out.println("Fatal: ");
    logger.fatal("FATAL message");
    System.out.println("Done\n====================");
  }

  /**
   * Returns process ID
   *
   * @return process ID
   */
  private static String getPID(){
    String name = ManagementFactory.getRuntimeMXBean().getName();
    Pattern p = Pattern.compile("(\\d+)@.+");
    Matcher m = p.matcher(name);
    if (m.matches()){
      return m.group(1);
    }
    return name;
  }

  /**
   * MBean interface definition
   */
  public static interface Log4JManagerMBean{

    /**
     * Setting log level to <code>TRACE</code>
     */
    public void setTraceLevel();

    /**
     * Setting log level to <code>DEBUG</code>
     */
    public void setDebugLevel();

    /**
     * Setting log level to <code>INFO</code>
     */
    public void setInfoLevel();

    /**
     * Setting log level to <code>WARN</code>
     */
    public void setWarnLevel();

    /**
     * Setting log level to <code>ERROR</code>
     */
    public void setErrorLevel();

    /**
     * Setting log level to <code>FATAL</code>
     */
    public void setFatalLevel();

  }

  /**
   * MBean implementation
   */
  public static class Log4JManager implements Log4JManagerMBean{

    /**
     * Returns instance of the logger for category <code>ru.skipy.logging</code>
     *
     * @return logger instance
     */
    private Logger getLogger(){
      return LogManager.getLogger("ru.skipy.logging");
    }

    /**
     * Setting log level to <code>TRACE</code>
     */
    @Override
    public void setTraceLevel() {
      getLogger().setLevel(Level.TRACE);
    }

    /**
     * Setting log level to <code>DEBUG</code>
     */
    @Override
    public void setDebugLevel() {
      getLogger().setLevel(Level.DEBUG);
    }

    /**
     * Setting log level to <code>INFO</code>
     */
    @Override
    public void setInfoLevel() {
      getLogger().setLevel(Level.INFO);
    }

    /**
     * Setting log level to <code>WARN</code>
     */
    @Override
    public void setWarnLevel() {
      getLogger().setLevel(Level.WARN);
    }

    /**
     * Setting log level to <code>ERROR</code>
     */
    @Override
    public void setErrorLevel() {
      getLogger().setLevel(Level.ERROR);
    }

    /**
     * Setting log level to <code>FATAL</code>
     */
    @Override
    public void setFatalLevel() {
      getLogger().setLevel(Level.FATAL);
    }
  }
}
